#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_test_assistant import WXTestAssistant, skip_nonwindows
from .. import push_button


@skip_nonwindows
class TestWxPushButton(WXTestAssistant, push_button.TestPushButton):
    """ WXPushButton tests. """

    def button_pressed(self):
        """ Press the button programmatically.

        """
        self.send_wx_event(self.widget, wx.EVT_LEFT_DOWN)

    def button_released(self):
        """ Release the button programmatically.

        """
        widget = self.widget
        self.send_wx_event(widget, wx.EVT_LEFT_UP)
        self.send_wx_event(widget, wx.EVT_LEAVE_WINDOW)

    def button_clicked(self):
        """ Click the button programmatically.

        """
        self.send_wx_event(self.widget, wx.EVT_BUTTON)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
""" A Lotka-Volterra model. 

This example plots a non-trivial system and allows the user to explore the
parameter space using sliders. The system is a first order Lotka-Volterra 
model, also known as a predator-prey model::

    http://en.wikipedia.org/wiki/Lotka%E2%80%93Volterra_equation

As in the sine plot example, the plot is on the left while the parameter 
sliders and the color controls are on the right. The plot will update as 
the slider values change.

This is the main driver for the example. The Enaml UI is defined in 
lotka_volterra_ui.enaml

This demo requires numpy and Chaco.

"""
import numpy as np

from traits.api import Array, HasTraits, Float, on_trait_change

import enaml


class LotkaVolterra(HasTraits):
    """ A simple Lotka-Volterra model.

    """

    dt = Float(1./128)
    total_time = Float(20.0)

    prey0 = Float(5.0)
    predator0 = Float(2.0)
    prey_growth = Float(1.0)
    predation_rate = Float(0.2)
    predator_growth = Float(0.04)
    predator_death = Float(0.5)

    t = Array()
    prey = Array()
    predator = Array()

    @on_trait_change('dt,total_time,prey0,predator0,prey_growth,predation_rate,predator_growth,predator_death')
    def integrate(self):
        """ Use a modifed Euler's method to integrate the ODE.
        """
        nsteps = int(round(self.total_time / self.dt))
        t = self.dt * np.arange(nsteps)
        x = self.prey0
        y = self.predator0
        prey = np.empty(nsteps)
        prey[0] = x
        predator = np.empty(nsteps)
        predator[0] = y
        for i in xrange(1, nsteps):
            x2 = x + self.dt * (self.prey_growth*x - self.predation_rate*x*y)
            y2 = y + self.dt * (-self.predator_death*y + self.predator_growth*x2*y)
            prey[i] = x2
            predator[i] = y2
            x = x2
            y = y2
        self.trait_set(
            t=t,
            prey=prey,
            predator=predator,
        )


def main():
    from simple_plot import SimplePlot
    SimplePlot.activate()

    with enaml.imports():
        from lotka_volterra_ui import Main

    model = LotkaVolterra()
    model.integrate()
    window = Main(model=model)
    window.show()


if __name__ == '__main__':
    main()




#------------------------------------------------------------------------------
#  Copyright (c) 2012, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import bisect

from .wx_image import WXImage

from ....noncomponents.abstract_icon import AbstractTkIcon


def _scale_size(from_size, to_size):
    """ An internal function used to scale sizes.

    Scales from_size to fit within the confines of to_size as closely as
    possible while maintaining the aspect ratio of from_size.

    Parameters
    ----------
    from_size : (width, height)
        The width, height tuple of integers of the scaling size.
    
    to_size : (width, height)
        The width, height tuple of integers of the bounding size.

    Returns
    -------
    result : (width, height)
        The width, heigh of the properly scaled size.
    
    """
    from_width, from_height = from_size
    to_width, to_height = to_size
    ratio = from_width / float(from_height)
    if abs(from_width - to_width) < abs(from_height - to_height):
        out_width = to_width
        out_height = int(out_width / ratio)
    else:
        out_height = to_height
        out_width = int(out_height * ratio)
    return (out_width, out_height)


class _IconEntry(object):
    """ An object which holds information about an icon. 

    Instances of this class are used internally by the WXIcon class to
    implement its icon data store.

    """
    def __init__(self, image, mode, state):
        """ Initialize an _IconEntry.

        Parameters
        ----------
        image : WXImage
            The WXImage instance held by this icon entry.
        
        mode : string
            The mode of the given image.
            
        state : string
            The state of the given image.
        
        """
        self.image = image
        self.mode = mode
        self.state = state

    def __lt__(self, other):
        """ Overridden '<' operator. The comparison is based on the 
        size of the entry and allows for instances to be sorted.

        """
        return self.size < other.size

    @property
    def size(self):
        """ Returns the (width, height) size of the underlying image.

        """
        return self.image.size

    @property
    def area(self):
        """ Returns the width * height area of the entry.

        """
        size = self.size
        return size[0] * size[1]


class WXIcon(AbstractTkIcon):
    """ A Wx implementation of AbstractTkIcon
    
    """
    def __init__(self):
        """ Initialize a WxIcon.

        """
        # The list of IconEntry instances for this icon. The are kept
        # in sorted order according to their size.
        self._entries = []

    #--------------------------------------------------------------------------
    # Private Methods
    #--------------------------------------------------------------------------
    def _match(self, size, mode, state):
        """ A private method which searches for a suitable _IconEntry
        for the given size, mode, and state. Returns None if no match
        can be made.

        """
        area = size[0] * size[1]
        match = None
        for entry in self._entries:
            if entry.mode == mode and entry.state == state:
                if entry.area >= area:
                    match = entry
                    break
        return match
    
    def _best_match(self, size, mode, state):
        """ A private method which returns the best _IconEntry match for
        the request, or None if no match can be made.

        """
        # Try the default match and then cycle through the various states
        # in an intelligent fashion trying to find any other match. This 
        # logic is more or less identical to QIcon's internal logic.
        match = self._match(size, mode, state)
        if match is None:
            opp_state = 'off' if state == 'on' else 'on'
            if mode == 'disabled' or mode == 'selected':
                opp_mode = 'selected' if mode == 'disabled' else 'disabled'
                new_info = (
                    ('normal', state),
                    ('active', state),
                    (mode, opp_state),
                    ('normal', opp_state),
                    ('active', opp_state),
                    (opp_mode, state),
                    (opp_mode, opp_state),
                )
            else:
                opp_mode = 'active' if mode == 'normal' else 'normal'
                new_info = (
                    (opp_mode, state),
                    (mode, opp_state),
                    (opp_mode, opp_state),
                    ('disabled', state),
                    ('selected', state),
                    ('disabled', opp_state),
                    ('selected', opp_state),
                )
            for new_mode, new_state in new_info:
                match = self._match(size, new_mode, new_state)
                if match is not None:
                    break
        return match

    #--------------------------------------------------------------------------
    # Abstract API Implementation
    #--------------------------------------------------------------------------
    @classmethod
    def from_file(cls, path):
        """ Create a new icon from a file on disk.

        Parameters
        ----------
        path : string
            The path to the image to use for the default mode and state.
            
        Returns
        -------
        result : AbstractTkIcon
            An new icon instance.

        """
        img = WXImage.from_file(path)
        return cls.from_image(img)
    
    @classmethod
    def from_image(cls, image):
        """ Create a new icon from an instance of AbstractTkImage using
        the default 'normal' and 'on' states.
                
        Parameters
        ----------
        image : AbstractTkImage
            An appropriate instance of AbstractTkImage to use for the
            default mode and state.

        Returns
        -------
        result : AbstractTkIcon
            An new icon instance.
            
        """
        if not isinstance(image, WXImage):
            msg = 'Image must be an instance of WXImage. Got %s instead.'
            raise TypeError(msg % type(image))
        new_icon = cls()
        new_icon.add_image(image, 'normal', 'on')
        return new_icon

    def get_image(self, size, mode='normal', state='on'):
        """ Get an appropriate image instance for the requested size, 
        mode and state.
        
        Parameters
        ----------
        size : (width, height)
            The size of the requested image. The returned image may
            be smaller, but will never be larger than this size.
            
        mode : string
            The mode of the requested image.
            
        state : string
            The state of the requested image.
        
        Returns
        -------
        result : AbstractTkImage
            An appropriate image instance.

        """
        match = self._best_match(size, mode, state)
        if match is None:
            res = WXImage()
        else:
            res = match.image
            actual_size = match.size
            if actual_size != size:
                scaled_size = _scale_size(actual_size, size)
                res = res.scale(scaled_size)
        return res

    def add_image(self, image, mode='normal', state='on'):
        """ Add an image instance for use by the icon with the given 
        mode and state.
        
        Parameters
        ----------
        image : AbstractTkImage
            An appropriate image instance.
            
        mode : string, optional
            The mode of the image. The default is 'normal'.
            
        state : string, optional
            The state of the image. The default is 'on'.
        
        """
        if not isinstance(image, WXImage):
            msg = 'Image must be an instance of WXImage. Got %s instead.'
            raise TypeError(msg % type(image))
        size = image.size
        match = self._match(size, mode, state)
        if match is not None:
            match.image = image
        else:
            entry = _IconEntry(image, mode, state)
            bisect.insort(self._entries, entry)

    def actual_size(self, size, mode='normal', state='on'):
        """ Returns the actual size for the requested size, mode, and
        state.

        The returned size may be smaller but will never be larger.

        Parameters
        ----------
        size : (width, height)
            The size of the requested image. The returned image may
            be smaller, but will never be larger than this size.
            
        mode : string, optional
            The mode of the requested image. The default is 'normal'.
            
        state : string, optional
            The state of the requested image. The default is 'on'.
        
        Returns
        -------
        result : (width, height)
            The actual size for the requested size. If no suitable
            match can be found, the result will be (-1, -1).

        """
        match = self._best_match(size, mode, state)
        if match is None:
            res = (-1, -1)
        else:
            res = match.size
            if res != size:
                res = _scale_size(res, size)
        return res
        
    def available_sizes(self, mode='normal', state='on'):
        """ Returns the available image sizes for the given mode and 
        state.
        
        Sizes other than these may be requested, but the implemetation 
        may scale down the image on the fly to the requested size.
        
        Parameters
        ----------
        mode : string, optional
            The requested image mode. The default is 'normal'.
            
        state : string, optional
            The requested image state. The default is 'on'.
        
        """
        sizes = []
        for entry in self._entries:
            if entry.mode == mode and entry.state == state:
                sizes.append(entry.size)
        return sizes




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from traits.api import HasTraits, Instance, DelegatesTo

from .style_sheet import StyleSheet
from .style_node import StyleNode
from .style_manager import StyleManager


class StyleMixin(HasTraits):

    style_sheet = Instance(StyleSheet, ())

    style_node = Instance(StyleNode, ())

    style_manager = Instance(StyleManager)

    def _style_manager_default(self):
        return StyleManager(component=self)

    #--------------------------------------------------------------------------
    # Style Value Delegates
    #--------------------------------------------------------------------------
    #: The alignment of a widget in its parent layout
    alignment = DelegatesTo('style_node')

    #: The foreground color of a widget. This is usually text color.
    fgcolor = DelegatesTo('style_node')

    #: The background color of a widget.
    bgcolor = DelegatesTo('style_node')

    #: The font to use for the widget.
    font = DelegatesTo('style_node')

    #: The margins to place around a layout.
    margins = DelegatesTo('style_node')

    ##: The suggested (w, h) size of the widget.
    #size_hint = DelegatesTo('style_node')

    #: How the widget should be resized when placed in a layout.
    size_policy = DelegatesTo('style_node')

    #: The interelement spacing in a layout
    spacing = DelegatesTo('style_node')

    #: The stretch factor to use when adding the widget to a layout
    stretch = DelegatesTo('style_node')

    #: The string name of the style class to use for this node.
    style_class = DelegatesTo('style_node')




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Instance
from enable.component import Component as EnableComponent

from .control import Control, AbstractTkControl


class AbstractTkEnableCanvas(AbstractTkControl):
    """ The abstract toolkit EnableCanvas interface.

    """
    @abstractmethod
    def shell_component_changed(self, component):
        """ The change handler for the 'component' attribute on the
        shell object.

        """
        raise NotImplementedError


class EnableCanvas(Control):
    """ An Enable canvas widget that will draw any Enable Component
    object.
    
    """
    #: The enable.component.Component instance to draw.
    component = Instance(EnableComponent)
    
    #: An EnableCanvas ignores its hug_width by default, so that it
    #: expands freely in width.
    hug_width = 'ignore'
    
    #: An Enable Canvas ignore its hug_height by default, so that it
    #: expands freely in height.
    hug_height = 'ignore'

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkEnableCanvas)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui
from .qt_control import QtControl

from ...components.html import AbstractTkHtml


class QtHtml(QtControl, AbstractTkHtml):
    """ A Qt implementation of Html.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying widget to display HTML.

        """
        self.widget = QtGui.QTextEdit(parent)

    def initialize(self):
        """ Initializes the attributes of the control.

        """
        super(QtHtml, self).initialize()
        self.widget.setReadOnly(True)
        self.set_page_source(self.shell_obj.source)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_source_changed(self, source):
        """ The change handler for the 'source' attribute.

        """
        self.set_page_source(source)

    def set_page_source(self, source):
        """ Sets the page source for the underlying control.

        """
        self.widget.setHtml(source)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtCore
from .qt_container import QtContainer
from .qt_resizing_widgets import QResizingGroupBox

from ...components.group_box import AbstractTkGroupBox
from ...layout.geometry import Box


QT_ALIGNMENTS = dict(
    left=QtCore.Qt.AlignLeft,
    right=QtCore.Qt.AlignRight,
    center=QtCore.Qt.AlignHCenter,
)


class QtGroupBox(QtContainer, AbstractTkGroupBox):
    """ A Qt4 implementation of GroupBox.

    """
    #: Don't use a widget item to compute the layout rect for a group
    #: box, it results in not enough space around/above neighbors.
    use_widget_item_for_layout = False
    
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QGroupBox control.

        """
        self.widget = QResizingGroupBox(parent)

    def initialize(self):
        """ Intializes the widget with the attributes of this instance.

        """
        super(QtGroupBox, self).initialize()
        shell = self.shell_obj
        self.set_title(shell.title)
        self.set_flat(shell.flat)
        self.set_title_align(shell.title_align)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_title_changed(self, title):
        """ Update the title of the group box with the new value from the
        shell object.

        """
        # We perform the title update in a relayout context to 
        # prevent flicker and multiple calls to relayout.
        self.shell_obj.request_relayout_task(self.set_title, title)

    def shell_flat_changed(self, flat):
        """ Update the flat flag of the group box with the new value from
        the shell object.

        """
        # We perform the flat update in a relayout context to 
        # prevent flicker and multiple calls to relayout.
        self.shell_obj.request_relayout_task(self.set_flat, flat)

    def shell_title_align_changed(self, align):
        """ Update the title alignment to the new value from the shell 
        object.

        """
        self.set_title_align(align)

    def get_contents_margins(self):
        """ Return the (top, left, right, bottom) margin values for the
        widget.

        """
        m = self.widget.contentsMargins()
        return Box(m.top(), m.right(), m.bottom(), m.left())

    #--------------------------------------------------------------------------
    # Widget Update methods 
    #--------------------------------------------------------------------------
    def set_title(self, title):
        """ Updates the title of group box.

        """
        self.widget.setTitle(title)
    
    def set_flat(self, flat):
        """ Updates the flattened appearance of the group box.

        """
        self.widget.setFlat(flat)
    
    def set_title_align(self, align):
        """ Updates the alignment of the title of the group box.

        """
        qt_align = QT_ALIGNMENTS[align]
        self.widget.setAlignment(qt_align)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_control import WXControl

from ...components.label import AbstractTkLabel


class WXLabel(WXControl, AbstractTkLabel):
    """ A wxPython implementation of Label.

    """
    #: The internal cached size hint which is used to determine whether
    #: of not a size hint updated event should be emitted when the text
    #: in the label changes
    _cached_size_hint = None

    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying wx.StaticText control.

        """
        self.widget = wx.StaticText(parent)

    def initialize(self):
        """ Initializes the attributes on the underlying control.

        """
        super(WXLabel, self).initialize()
        self.set_text(self.shell_obj.text)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_text_changed(self, text):
        """ The change handler for the 'text' attribute on the shell 
        component.

        """
        self.set_text(text)

    def shell_word_wrap_changed(self, wrap):
        """ The change handler for the 'word_wrap' attribute on the 
        shell component. Wx does not support word wrap, so this is 
        a no-op.

        """
        pass

    def set_text(self, text):
        """ Sets the text on the underlying control.

        """
        self.widget.SetLabel(text)

        # Emit a size hint updated event if the size hint has actually
        # changed. This is an optimization so that a constraints update
        # only occurs when the size hint has actually changed. This 
        # logic must be implemented here so that the label has been
        # updated before the new size hint is computed. Placing this
        # logic on the shell object would not guarantee that the label
        # has been updated at the time the change handler is called.
        cached = self._cached_size_hint
        hint = self._cached_size_hint = self.size_hint()
        if cached != hint:
            self.shell_obj.size_hint_updated()
    



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from datetime import datetime
import os

import numpy as np

from chaco.api import Plot, ArrayPlotData

import enaml
from enaml.item_models.abstract_item_model import AbstractItemModel


class Node(object):
    """ A simple node class for building up a filesystem tree.

    """
    def __init__(self, parent, text, stat, files=None, children=None):
        self.parent = parent
        self.text = text
        self.stat = stat
        self.children = children or []


def make_fs_tree():
    """ Returns the root node of a filesystem tree for the directory 
    downward.

    """
    this_path = os.path.abspath(__file__)
    root_path = os.path.split(os.path.dirname(this_path))[0]
    node_cache = {}
    root = Node(None, '', '')
    node_cache[root_path] = root
    gen = os.walk(root_path)

    while True:
        try:
            dirpath, dirnames, filenames = gen.next()
        except StopIteration:
            break
        if dirpath not in node_cache:
            continue
        parent = node_cache[dirpath]
        for dn in dirnames:
            if not dn.startswith('.'):
                fd = os.path.join(dirpath, dn)
                s = os.stat(fd)
                node = Node(parent, dn, s)
                node_cache[fd] = node
                parent.children.append(node)
        for fn in filenames:
            if not fn.startswith('.'):
                s = os.stat(os.path.join(dirpath, fn))
                parent.children.append(Node(parent, fn, s))

    return root


class FSModel(AbstractItemModel):
    """ An AbstractItemModel implementation which exposes the
    filesystem tree.

    """
    fs_root = make_fs_tree()

    def index(self, row, column, parent=None):
        """ Abstract method implementation that will create a valid
        model index for the given row and column if they are valid.
        Otherwise, returns an invalid index.

        """
        if self.has_index(row, column, parent):
            if parent is None:
                parent_node = self.fs_root
            else:
                parent_node = parent.context
            child = parent_node.children[row]
            return self.create_index(row, column, child)

    def parent(self, index):
        """ Abstract method implementation that always returns None, 
        forcing the model to be flat.

        """
        if index is None:
            return
        node = index.context
        node_parent = node.parent
        if node_parent is self.fs_root:
            return None
        node_parent_parent = node.parent.parent
        idx = node_parent_parent.children.index(node_parent)
        return self.create_index(idx, index.column, node_parent)
    
    def row_count(self, idx):
        if idx is None:
            node = self.fs_root
        else:
            node = idx.context
        return len(node.children)
    
    def column_count(self, idx):
        return 3
    
    def data(self, idx):
        col = idx.column
        node = idx.context
        if col == 0:
            res = node.text
        elif col == 1:
            res = str(node.stat.st_size / 1024) + ' kb'
        else:
            dt = datetime.fromtimestamp(node.stat.st_mtime)
            res = dt.strftime('%c')
        return res
    
    def horizontal_header_data(self, section):
        return ['Name', 'Size', 'Last Modified'][section]


def make_sin_plot():
    """ Returns a chaco plot which plots a simple sin curve.

    """
    idx = np.linspace(0, np.pi * 2, 100)
    val = np.sin(idx)
    plt = Plot(ArrayPlotData(x=idx, y=val), padding_top=30, padding_bottom=30, 
               padding_left=35, padding_right=10, title='Sin Plot',
               fill_padding=False, background='transparent')
    plt.plot(('x', 'y'), color='red')
    return plt


def make_html_table():
    """ Returns a string of html table data.

    """
    html = """
    <html>
        <h1> Simple HTML </h1>
        <body>
            <h4>Cell that spans two columns:</h4>
            <table border="1">
                <tr>
                  <th>Name</th>
                  <th colspan="2">Telephone</th>
                </tr>
                <tr>
                  <td>Tony Stark</td>
                  <td>555 77 854</td>
                  <td>555 77 855</td>
                </tr>
            </table>
            <h4>Cell that spans two rows:</h4>
            <table border="1">
                <tr>
                  <th>First Name:</th>
                  <td>Pepper Potts</td>
                </tr>
                <tr>
                  <th rowspan="2">Telephone:</th>
                  <td>555 77 854</td>
                </tr>
                <tr>
                  <td>555 77 855</td>
                </tr>
            </table>
        </body>
    </html>
    """
    return html


def main():
    with enaml.imports():
        from widget_gallery import WidgetGallery

    img_path = os.path.join(os.path.dirname(__file__), 'lena.png')
    view = WidgetGallery(
        fs_model=FSModel(), sin_plot=make_sin_plot(),
        html_data=make_html_table(), img_path=img_path,
    )
    view.show()


if __name__ == '__main__':
    main()




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from traits.api import Instance, Str

from casuarius import ConstraintVariable

from .container import AbstractTkContainer, Container

from ..layout.layout_helpers import align, hbox, vbox


class AbstractTkForm(AbstractTkContainer):
    """ The abstract toolkit Form interface.

    """
    pass


class Form(Container):
    """ A Container subclass that arranges its layout children as a two 
    column form.

    The left column is typically Labels (but this is not a requirement).
    The right are the actual widgets for data entry. The children should
    be in alternating label/widget order. If there are an odd number
    of children, the last child will span both columns.

    """
    #: The ConstraintVariable giving the midline along which the labels
    #: and widgets are aligned.
    midline = Instance(ConstraintVariable)
    def _midline_default(self):
        label = 'midline_{0}_{1:x}'.format(type(self).__name__, id(self))
        return ConstraintVariable(label)

    #: The strength for the form layout constraints.
    # FIXME: Use an Enum.
    layout_strength = Str('strong')

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkForm)
    
    def component_constraints(self):
        """ Computes the current form constraints for the current
        children.

        """
        # FIXME: do something sensible when children are not visible.
        children = self.constraints_children
        labels = children[::2]
        widgets = children[1::2]

        n_labels = len(labels)
        n_widgets = len(widgets)

        if n_labels != n_widgets:
            if n_labels > n_widgets:
                odd_child = labels.pop()
            else:
                odd_child = widgets.pop()
        else:
            odd_child = None

        layout_strength = self.layout_strength
        constraints = []

        # Align the left side of each widget with the midline constraint
        # variable of the form.
        midline = self.midline
        for widget in widgets:
            cn = (widget.left == midline) | layout_strength
            constraints.append(cn)

        # Arrange each label/widget pair horizontally in the form
        labels_widgets = zip(labels, widgets)
        vbox_args = [hbox(label, widget) for label, widget in labels_widgets]
        if odd_child is not None:
            vbox_args.append(odd_child)
        constraints.append(vbox(*vbox_args))

        for label, widget in labels_widgets:
            # FIXME: baselines would be much better.
            constraints.append(align('v_center', label, widget) | layout_strength)

        return constraints

    def default_user_constraints(self):
        """ Overridden parent class method which returns an empty list.
        All constraints are supplied by 'component_constraints()'.

        """
        return []




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from contextlib import contextmanager
from traits.api import push_exception_handler, pop_exception_handler

class EnamlError(Exception):
    """ A general exception used to indicate an error with Enaml or 
    the Enaml runtime that doesn't fit the semantics of any other 
    Python standard exception.

    """
    pass


class EnamlSyntaxError(EnamlError):
    """ Derived from EnamlError and used to indicate an Error in the
    syntax of a .enaml file.

    """
    pass


class EnamlRuntimeError(EnamlError):
    """ An EnamlError that occurs during the execution of an Enaml 
    script that couldn't be caught at compile time but prevents the 
    runtime from continuing to execute the script.

    """
    pass


def null_handler(obj, trait, old, new):
    """ A null Traits exception handler.
    
    """
    pass

@contextmanager
def notification_context(handler=null_handler, reraise_exceptions=True, main=False, locked=True):
    """ Context manager to temporarily add a TraitsExceptionHandler
    
    We use a context manager to ensure that the exception handler gets cleared
    no matter what.  Default behaviour is to use the null_handler with
    exceptions re-raised, which means any exceptions which occur will be passed
    through.
    
    """
    yield push_exception_handler(handler, reraise_exceptions, main, locked)
    pop_exception_handler()


class ShellExceptionContext(object):
    """ Context manager to that manages error state of shell objects.
    
    Any exceptions which occur within a with statement using this context
    will get swallowed and set into the exception trait of the shell object.
    
    """
    def __init__(self, shell_obj):
        self.shell_obj = shell_obj
    
    def __enter__(self):
        return self
    
    def __exit__(self, exc_type, exc_value, traceback):
        self.shell_obj.exception = exc_value
        self.shell_obj.error = (exc_value is not None)
        if exc_value is not None:
            control_exception_handler = self.shell_obj.toolkit.control_exception_handler
            if control_exception_handler is not None:
                control_exception_handler(exc_value)
        return True


class ShellNotificationContext(ShellExceptionContext):
    """ Combination of a ShellExceptionContext and a notification_context
    
    Default behaviour is to use the null_handler with exceptions re-raised,
    which means any exceptions which occur during traits notification will be
    caught and set to the shell object's exception trait.
    
    """
    def __init__(self, shell_obj, handler=null_handler, reraise_exceptions=True,
            main=False, locked=False):
        self.shell_obj = shell_obj
        self.handler = handler
        self.reraise_exceptions = reraise_exceptions
        self.main = main
        self.locked = locked
    
    def __enter__(self):
        self.notification_handler = push_exception_handler(self.handler,
            self.reraise_exceptions, self.main, self.locked)
        return self
    
    def __exit__(self, exc_type, exc_value, traceback):
        pop_exception_handler()
        return super(ShellNotificationContext, self).__exit__(exc_type, exc_value, traceback)
    


#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt_test_assistant import QtTestAssistant

from .. import datetime_edit

from enaml.backends.qt.qt_datetime_edit import qdatetime_to_python


class TestQtDatetimeEdit(QtTestAssistant, datetime_edit.TestDatetimeEdit):
    """ QtDateEdit tests. 

    """
    def get_datetime(self, widget):
        """  Get the toolkits widget's active datetime.

        """
        datetime = widget.dateTime()
        return qdatetime_to_python(datetime)

    def get_min_datetime(self, widget):
        """  Get the toolkits widget's maximum datetime attribute.

        """
        datetime = widget.minimumDateTime()
        return qdatetime_to_python(datetime)

    def get_max_datetime(self, widget):
        """ Get the toolkits widget's minimum datetime attribute.

        """
        datetime = widget.maximumDateTime()
        return qdatetime_to_python(datetime)

    def change_datetime(self, widget, datetime):
        """ Simulate a change datetime action at the toolkit widget.

        """
        # The setDateTime method in Qt will also emit the
        # dateTimeChanged signal.
        self.widget.setDateTime(datetime)

    def get_datetime_as_string(self, widget):
        """  Get the toolkits widget's active datetime as a string.

        """
        return self.widget.text()




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .wx_container import WXContainer

from ...components.form import AbstractTkForm


class WXForm(WXContainer, AbstractTkForm):
    """ A wxPython implementation of Container.

    """
    # The WXContainer implementation is enough.
    pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui, QtCore

from .qt_bounded_datetime import QtBoundedDatetime

from ...components.datetime_edit import AbstractTkDatetimeEdit
from ...guard import guard


# Workaround for an incompatibility between PySide and PyQt
try:
    qdatetime_to_python = QtCore.QDateTime.toPython
except AttributeError:
    qdatetime_to_python = QtCore.QDateTime.toPyDateTime


class QtDatetimeEdit(QtBoundedDatetime, AbstractTkDatetimeEdit):
    """ A Qt implementation of DateTimeEdit.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QtCalendarWidget.

        """
        self.widget = QtGui.QDateTimeEdit(parent)

    def initialize(self):
        """ Initializes the attributes of the control.

        """
        super(QtDatetimeEdit, self).initialize()
        self.set_format(self.shell_obj.datetime_format)
    
    def bind(self):
        """ Connects the signal handlers for the date edit widget. Not
        meant for public consumption.

        """
        super(QtDatetimeEdit, self).bind()
        self.widget.dateTimeChanged.connect(self.on_datetime_changed)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_datetime_format_changed(self, datetime_format):
        """ The change handler for the 'format' attribute.

        """
        self.set_format(datetime_format)

    def on_datetime_changed(self):
        """ The signal handler for the controls's changed event. Not
        meant for public consumption.

        """
        # only emit update the shell object if the widget was 
        # changed via the ui and not programmatically.
        if not guard.guarded(self, 'updating'):
            shell = self.shell_obj
            qdatetime = self.widget.dateTime()
            new_datetime = qdatetime_to_python(qdatetime)
            shell.datetime = new_datetime
            shell.datetime_changed(new_datetime)

    def set_datetime(self, datetime):
        """ Sets and the datetime on the widget.

        """
        # Calling setDate will trigger the dateChanged signal.
        # We want to avoid that feeback loop since the value is
        # being set programatically.
        with guard(self, 'updating'):
            self.widget.setDateTime(datetime)

    def set_min_datetime(self, datetime):
        """ Sets the minimum datetime on the widget with the provided 
        value.

        """
        # Calling setMinimumDateTime *may* trigger the dateTimeChanged 
        # signal, if the datetime needs to be clipped. We want to avoid 
        # that feeback  loop since the value is being set programatically 
        # and the new datetime will already have been updated by the shell 
        # object.
        with guard(self, 'updating'):
            self.widget.setMinimumDateTime(datetime)

    def set_max_datetime(self, datetime):
        """ Sets the maximum datetime on the widget with the provided 
        value.

        """
        # Calling setMaximumDateTime *may* trigger the dateTimeChanged 
        # signal, if the datetime needs to be clipped. We want to avoid 
        # that feeback  loop since the value is being set programatically 
        # and the new datetime will already have been updated by the shell 
        # object.
        with guard(self, 'updating'):
            self.widget.setMaximumDateTime(datetime)
        
    def set_format(self, datetime_format):
        """ Sets the display format on the widget with the provided 
        value.

        """
        self.widget.setDisplayFormat(datetime_format)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import weakref

import wx

from .wx_constraints_widget import WXConstraintsWidget

from ...components.container import AbstractTkContainer


class _WXPanel(wx.Panel):
    """ A subclass of wxPanel which calls back onto the shell Container 
    to get the size hint. If that size hint is invalid, it falls back 
    onto the Wx default.

    """
    def __init__(self, wx_container, *args, **kwargs):
        super(_WXPanel, self).__init__(*args, **kwargs)
        self.wx_container = weakref.ref(wx_container)

    def GetBestSizeTuple(self):
        """ Computes the size hint from the given Container, falling
        back on the default size hint computation if the Container 
        returns one that is invalid.

        """
        res = None
        wx_container = self.wx_container()
        if wx_container is not None:
            shell = wx_container.shell_obj
            if shell is not None:
                sh = shell.size_hint()
                if sh != (-1, -1):
                    res = sh
        if res is None:
            res = super(_WXPanel, self).GetBestSizeTuple()
        return res


class WXContainer(WXConstraintsWidget, AbstractTkContainer):
    """ A Wx implementation of a Container.

    """
    def create(self, parent):
        """ Creates the underlying Wx widget.

        """
        self.widget = _WXPanel(self, parent)

    def bind(self):
        """ Bind the widget to the wx.EVT_SIZE.

        """
        super(WXContainer, self).bind()
        self.widget.Bind(wx.EVT_SIZE, self.on_resize)
    
    def on_resize(self, event):
        """ Triggers a relayout of the shell object since the component
        has been resized.

        """
        # Notice that we are calling refresh() here instead of 
        # request_refresh() since we want the refresh to happen
        # immediately. Otherwise the resize layouts will appear 
        # to lag in the ui. This is a safe operation since by the
        # time we get this resize event, the widget has already 
        # changed size. Further, the only geometry that gets set
        # by the layout manager is that of our children. And should
        # it be required to resize this widget from within the layout
        # call, then the layout manager will do that asynchronously.
        self.shell_obj.refresh()

        # We need to call event.Skip() here or certain controls won't
        # won't inform their children to resize.
        event.Skip()




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import locale
import unittest

from enaml.localization import (
    SystemLocale, get_default_locale, set_default_locale,
)


class TestDefaultLocale(unittest.TestCase):

    def setUp(self):
        self.old = get_default_locale()
        set_default_locale(None)
    
    def tearDown(self):
        set_default_locale(self.old)

    def test_system_default_locale(self):
        l = get_default_locale()
        self.assertIsInstance(l, SystemLocale)

    def test_system_locale_types(self):
        l = get_default_locale()
        name, enc = locale.getlocale()
        self.assertEqual(l.name, name)
        self.assertEqual(l.encoding, enc)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import ABCMeta, abstractmethod

from ..localization import get_default_locale
from ..localization.abstract_locale import AbstractLocale


class AbstractValidator(object):
    """ An abstract base class defining the behavior of validators.

    Validators are used to check the text input by the user is valid
    for conversion, perform that conversion when requested, and to
    format a user value to unicode string for display.

    """
    __metaclass__ = ABCMeta

    #: A constant representing a clearly invalid result. For example,
    #: the text 'asdf' is clearly invalid when a number string is
    #: expected.
    INVALID = 0

    #: A constant which represents a string which is in a plausible
    #: intermediate state on its way to being valid. For example the
    #: string '123' is intermediate if a 4 digit number string is 
    #: expected.
    INTERMEDIATE = 1

    #: A constant which represents a string that is acceptable inpute 
    #: for the current context.
    ACCEPTABLE = 2

    def __init__(self, locale=None):
        """ Initialize an AbstractValidator.

        Parameters
        ----------
        locale : AbstractLocale or None, optional
            If provided, a Locale object which may assist in converting
            to/from the string value. Whether or not the locale is used
            is dependent on the given subclass.

        """
        self.locale = locale
    
    def _get_locale(self):
        """ The property getter for the 'locale' attribute. This returns
        the current locale, or retrieves the default locale if one has 
        not yet been assigned to the validator.

        """
        locale = self._locale
        if locale is None:
            locale = self._locale = get_default_locale()
        return locale
    
    def _set_locale(self, value):
        """ The property setter for the 'locale' attribute.

        """
        if value is not None and not isinstance(value, AbstractLocale):
            msg = ('locale object must be an instance of AbstractLocale or '
                   'None. Got %s instead.')
            raise TypeError(msg % type(value))
        self._locale = value
    
    #: A property which returns the locale object for this validator.
    #: If one is not explicitly provided, the framework default locale
    #: object will be used.
    locale = property(_get_locale, _set_locale)

    @abstractmethod
    def validate(self, text):
        """ Validate the input text for the given context. 

        This is an abstract method and must be implemented by subclasses.
        This method should analyze the user text and return one of three
        class attribute constants: INVALID, INTERMEDIATE, or ACCEPTABLE. 
        These constants have the following semantic meanings:
            
            INVALID : The text is clearly invalid, and the addition of
                more characters cannot make it valid. Example: the text
                'asdf' is invalid when a number string is expected.

            INTERMEDIATE : The text is a plausible value that can be made
                valid with the addition of more text. Example: the text
                '123' is intermediate when a 4 digit number string is 
                expected.

            ACCEPTABLE : The text is an acceptable value. 

        Parameters
        ----------
        text : unicode string
            The input string supplied by the user.
        
        Returns
        -------
        result : int
            One of the class attribute constants INVALID, INTERMEDIATE, 
            or ACCEPTABLE.
        
        Notes
        -----
        This method will be called with high frequency. Implementations
        should therefore take care to make sure that the method executes
        quickly.

        """
        raise NotImplementedError
    
    @abstractmethod
    def convert(self, text):
        """ Convert the input text into an appropriate value.

        This is an abstract method and must be implemented by subclasses.
        This method is used to convert from the user input text into a
        context-dependent value. This method will only be called if the
        'validate' method returned an ACCEPTABLE result. When this method 
        is called is determined by the owner of the validator and is not
        necessarily immediately after a call to 'validate'.

        If the input text cannot be properly converted, a ValueError
        should be raised.

        Parameters
        ----------
        text : unicode string
            The input string supplied by the user.

        Returns
        -------
        result : object
            An appropriately converted object.

        """
        raise NotImplementedError

    @abstractmethod
    def format(self, value):
        """ Convert the input value into a string which is appropriate
        for display.

        This is an abstract method and must be implemented by subclasses.
        This method is used to convert from a context dependent model
        value into a unicode string suitable for display. This method
        will be called whenever a model value changes and the display
        should be updated. This method should always succeed.

        Parameters
        ----------
        value : object
            The context-dependent input value.

        Returns
        -------
        result : unicode string
            A formatted unicode string for display.

        """
        raise NotImplementedError

    def normalize(self, text):
        """ Attempt to convert the given input text into a valid unicode
        string according to the rules of the validator.

        This method may be called by various controls at various times,
        such as when the control loses focus. This provides a chance for
        the programmer to coerce the text into a valid string or to fix
        the formatting of the user string. It is not necessary to return
        a valid value from this method, as the returned value will be 
        tested again for validity. The default implementation of this 
        method does nothing and returns the input string.

        If the method cannot properly normalize the string, the original
        text should be returned.

        Parameters
        ----------
        text : unicode string
            The input string supplied by the user.

        Returns
        -------
        result : unicode string
            The string to use in the control in lieu of the user supplied
            string.
        
        """
        return text




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Bool, Str, Instance, Property

from .control import Control, AbstractTkControl

from ..core.trait_types import EnamlEvent


class AbstractTkToggleControl(AbstractTkControl):
        
    @abstractmethod
    def shell_checked_changed(self, checked):
        raise NotImplementedError
    
    @abstractmethod
    def shell_text_changed(self, text):
        raise NotImplementedError


class ToggleControl(Control):
    """ An abstract toggle element. 
    
    An element which toggles the value of a boolean field. This is an 
    abstract class which should not be used directly. Rather, it provides
    common functionality for subclasses such as RadioButton, CheckBox and
    ToggleButton.

    """
    #: Whether the element is currently checked.
    checked = Bool

    #: A read only property which indicates whether the user is 
    #: currently pressing the element.
    down = Property(Bool, depends_on='_down')

    #: The text to use as the element's label.
    text = Str
    
    #: Fired when the element is toggled.
    toggled = EnamlEvent
    
    #: Fired when the element is pressed.
    pressed = EnamlEvent

    #: Fired when the element is released.
    released = EnamlEvent

    #: Internal storage for the down attribute
    _down = Bool
    
    #: How strongly a component hugs it's contents' width.
    #: Toggles hug their contents' width weakly by default.
    hug_width = 'weak'

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkToggleControl)

    #--------------------------------------------------------------------------
    # Property methods 
    #--------------------------------------------------------------------------
    def _get_down(self):
        """ The property getter for the 'down' attribute.

        """
        return self._down




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from ...components.abstract_application import AbstractTkApplication


class WXApplication(AbstractTkApplication):
    """ A Wx implementation of AbstractTkApplication.

    """
    def initialize(self, *args, **kwargs):
        """ Initializes the underlying wxApp object. It does *not* 
        start the event loop. If the application object is already 
        created, it is a no-op.

        """     
        app = wx.GetApp()
        if app is None:
            app = wx.PySimpleApp(*args, **kwargs)
        self._wx_app = app

    def start_event_loop(self):
        """ Starts the underlying application object's event loop, or 
        does nothing if it is already started. A RuntimeError will be 
        raised if the application object is not yet created.

        """
        app = wx.GetApp()
        if app is None:
            msg = 'Cannot start event loop. Application object not created.'
            raise RuntimeError(msg)
        
        if not app.IsMainLoopRunning():
            app.MainLoop()
                
    def event_loop_running(self):
        """ Returns True if the main event loop is running, False 
        otherwise.

        """
        app = wx.GetApp()
        if app is None:
            return False
        return app.IsMainLoopRunning()

    def app_object(self):
        """ Returns the underlying application object, or None if one 
        does not exist.

        """
        return wx.GetApp()

    def is_main_thread(self):
        """ Return True if this method was called from the main event
        thread, False otherwise.

        """
        app = wx.GetApp()
        if app is None:
            raise RuntimeError('Application object not yet created')
        return wx.Thread_IsMain()

    def call_on_main(self, callback, *args, **kwargs):
        """ Invoke the given callable in the main gui event thread at 
        some point in the future.

        Parameters
        ----------
        callback : callable
            The callable object to execute at some point in the future.

        *args
            Any positional arguments to pass to the callback.

        **kwargs
            Any keyword arguments to pass to the callback.

        """
        wx.CallAfter(callback, *args, **kwargs)

    def timer(self, ms, callback, *args, **kwargs):
        """ Invoke the given callable in the main gui event thread at 
        the given number of milliseconds in the future.

        Parameters
        ----------
        ms : int
            The number of milliseconds in the future to invoke the
            callable.

        callback : callable
            The callable object to execute at some point in the future.

        *args
            Any positional arguments to pass to the callback.

        **kwargs
            Any keyword arguments to pass to the callback.

        """
        wx.CallLater(ms, callback, *args, **kwargs)

    def process_events(self):
        """ Process all of the pending events in the event queue.

        """
        wx.YieldIfNeeded()




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Instance, Str

from .bounded_date import BoundedDate, AbstractTkBoundedDate

from ..core.trait_types import EnamlEvent


class AbstractTkDateEdit(AbstractTkBoundedDate):
    """ The abstract toolkit DateEdit interface.

    """
    @abstractmethod
    def shell_date_format_changed(self, date_format):
        """ The change handler for the 'date_format' attribute on the
        shell object.

        """
        raise NotImplementedError


class DateEdit(BoundedDate):
    """ A widget to edit a Python datetime.date object.

    A DateEdit displays a Python datetime.date using an appropriate
    toolkit specific control. This is a geometrically smaller control 
    than what is provided by Calendar.

    """
    #: A python date format string to format the date. If none is
    #: supplied (or is invalid) the system locale setting is used.
    #: This may not be supported by all backends.
    date_format = Str

    #: Triggered whenever the user changes the date through the ui
    #: control, but not programmatically. The event payload will be 
    #: the date on the control.
    date_changed = EnamlEvent
    
    #: How strongly a component hugs it's contents' width. DateEdits 
    #: ignore the width hug by default, so they expand freely in width.
    hug_width = 'ignore'

    #: Overridden parent class trait.
    abstract_obj = Instance(AbstractTkDateEdit)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt_control import QtControl
from .qt import QtGui

from ...components.progress_bar import AbstractTkProgressBar


class QtProgressBar(QtControl, AbstractTkProgressBar):
    """ Qt implementation of ProgressBar.

    """
    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QProgressBar.

        """
        self.widget = QtGui.QProgressBar(parent)
    
    def initialize(self):
        """ Initialize the attributes of the progress bar.

        """
        super(QtControl, self).initialize()
        self.widget.setTextVisible(False)
        shell = self.shell_obj
        self._set_minimum(shell.minimum)
        self._set_maximum(shell.maximum)
        self._set_value(shell.value)

    #--------------------------------------------------------------------------
    # Abstract Implementation Methods
    #--------------------------------------------------------------------------
    def shell_value_changed(self, value):
        """ The change handler for the 'value' attribute of the shell
        object.

        """
        self._set_value(value)

    def shell_minimum_changed(self, minimum):
        """ The change handler for the 'minimum' attribute of the shell
        object.

        """
        self._set_minimum(minimum)
            
    def shell_maximum_changed(self, maximum):
        """ The change handler for the 'maximum' attribute of the shell
        object

        """
        self._set_maximum(maximum)

    #--------------------------------------------------------------------------
    # Widget Update Methods
    #--------------------------------------------------------------------------
    def _set_value(self, value):
        """ Sets the value of the progress bar.

        """
        self.widget.setValue(value)

    def _set_minimum(self, minimum):
        """ Sets the minimum value of the progress bar.

        """
        self.widget.setMinimum(minimum)
    
    def _set_maximum(self, maximum):
        """ Sets the maximum value of the progress bar.

        """
        self.widget.setMaximum(maximum)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------

from __future__ import with_statement

import numpy as np

from enable.api import ColorTrait, Component
from traits.api import Float, Instance, Int
from chaco.api import AbstractOverlay, BaseXYPlot


class ZoomOverlay(AbstractOverlay):
    """
    Draws a trapezoidal selection overlay from the source plot to the
    destination plot.  Assumes that the source plot lies above the destination
    plot.
    """
    source = Instance(BaseXYPlot)
    destination = Instance(Component)
    other = Instance(Component)

    border_color = ColorTrait((0, 0, 0.7, 1))
    border_width = Int(1)
    fill_color = ColorTrait("dodgerblue")
    alpha = Float(0.3)

    def calculate_points(self, component):
        """
        Calculate the overlay polygon based on the selection and the location
        of the source and destination plots.
        """
        # find selection range on source plot
        x_start, x_end = self._get_selection_screencoords()
        if x_start > x_end:
            x_start, x_end = x_end, x_start

        y_end = self.source.y
        y_start = self.source.y2

        left_top = np.array([x_start, y_start])
        left_mid = np.array([x_start, y_end])
        right_top = np.array([x_end, y_start])
        right_mid = np.array([x_end, y_end])

        # Offset y because we want to avoid overlapping the trapezoid with the topmost
        # pixels of the destination plot.
        y = self.destination.y2 + 1

        left_end = np.array([self.destination.x, y])
        right_end = np.array([self.destination.x2, y])

        polygon = np.array((left_top, left_mid, left_end,
                         right_end,right_mid, right_top))
        left_line = np.array((left_top, left_mid, left_end))
        right_line = np.array((right_end,right_mid, right_top))

        return left_line, right_line, polygon

    def overlay(self, component, gc, view_bounds=None, mode="normal"):
        """
        Draws this overlay onto 'component', rendering onto 'gc'.
        """

        tmp = self._get_selection_screencoords()
        if tmp is None:
            return

        left_line, right_line, polygon = self.calculate_points(component)

        with gc:
            gc.translate_ctm(*component.position)
            gc.set_alpha(self.alpha)
            gc.set_fill_color(self.fill_color_)
            gc.set_line_width(self.border_width)
            gc.set_stroke_color(self.border_color_)
            gc.begin_path()
            gc.lines(polygon)
            gc.fill_path()

            gc.begin_path()
            gc.lines(left_line)
            gc.lines(right_line)
            gc.stroke_path()

        return

    def _get_selection_screencoords(self):
        """
        Returns a tuple of (x1, x2) screen space coordinates of the start
        and end selection points.  If there is no current selection, then
        returns None.
        """
        selection = self.source.index.metadata["selections"]
        if selection is not None and len(selection) == 2:
            mapper = self.source.index_mapper
            return mapper.map_screen(np.array(selection))
        else:
            return None

    #------------------------------------------------------------------------
    # Trait event handlers
    #------------------------------------------------------------------------

    def _source_changed(self, old, new):
        if old is not None and old.controller is not None:
            old.controller.on_trait_change(self._selection_update_handler, "selection",
                                           remove=True)
        if new is not None and new.controller is not None:
            new.controller.on_trait_change(self._selection_update_handler, "selection")
        return

    def _selection_update_handler(self, value):
        if value is not None and self.destination is not None:
            r = self.destination.index_mapper.range
            start, end = np.amin(value), np.amax(value)
            r.low = start
            r.high = end

            start_idx, end_idx = np.searchsorted(self.destination.index.get_data(), [start, end])
            value = self.destination.value.get_data()[start_idx:end_idx]
            if len(value):
                r = self.destination.value_mapper.range
                start, end = np.min(value), np.max(value)
                r.low = start
                r.high = end

            value = self.other.value.get_data()[start_idx:end_idx]
            if len(value):
                r = self.other.value_mapper.range
                start, end = np.min(value), np.max(value)
                r.low = start
                r.high = end
        else:
            self.destination.index_mapper.range.low = 'auto'
            self.destination.index_mapper.range.high = 'auto'
            self.destination.value_mapper.range.low = 'auto'
            self.destination.value_mapper.range.high = 'auto'
            self.other.value_mapper.range.low = 'auto'
            self.other.value_mapper.range.high = 'auto'

        self.source.request_redraw()
        self.destination.request_redraw()
        self.other.request_redraw()



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import wx

from .wx_constraints_widget import WXConstraintsWidget

from ...components.tab_group import AbstractTkTabGroup


#: A mapping from TabPosition enum values to qt tab positions.
_TAB_POSITION_MAP = {
    'top': wx.NB_TOP,
    'bottom': wx.NB_BOTTOM,
    'left': wx.NB_LEFT,
    'right': wx.NB_RIGHT,
}


class WXTabGroup(WXConstraintsWidget, AbstractTkTabGroup):
    """ A wx implementation of the Tabbed container.

    """
    #--------------------------------------------------------------------------
    # Setup Methods 
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Create the underlying wxNotebook control.

        """
        # Changing the tab position of the notebook dynamically is not
        # supported by wx (the rendering gets all messed up). So, the
        # tab position must be set at creation time.
        style = _TAB_POSITION_MAP[self.shell_obj.tab_position]
        self.widget = wx.Notebook(parent, style=style)
    
    def initialize(self):
        """ Initialize the attributes of the wxNotebook.

        """
        super(WXTabGroup, self).initialize()
        self.update_tabs()
        self.set_selected_index(self.shell_obj.selected_index)

    def bind(self):
        """ Bind to the events emitted by the underlying control.

        """
        super(WXTabGroup, self).bind()
        self.widget.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self._on_page_changed)

    #--------------------------------------------------------------------------
    # Implementation 
    #--------------------------------------------------------------------------
    def shell_tabs_changed(self, tabs):
        """ The change handler for the 'tabs' attribute of the shell 
        object.

        """
        self.update_tabs()

    def shell_tab_position_changed(self, tab_position):
        """ The change handler for the 'tab_position' attribute of the
        shell object.

        """
        # Changing tab position dynamically on wx is not supported.
        pass

    def shell_selected_index_changed(self, index):
        """ Update the widget index with the new value from the shell 
        object.

        """
        self.set_selected_index(index)

    def size_hint(self):
        """ Returns a (width, height) tuple of integers which represent
        the suggested size of the widget for its current state. This
        value is used by the layout manager to determine how much
        space to allocate the widget.

        """
        # XXX - This size hint computation needs some work. Need to 
        # look at the source for wxNotebook::GetBestSize and see what
        # is does and compare against QTabWidget::sizeHint. Like always,
        # the Qt implementation does the right thing and wx does not.
        widget = self.widget
        shell = self.shell_obj
        curr_shell = shell.selected_tab

        if curr_shell is None:
            # QTabWidget default for no tabs
            return (6, 6)
        
        size_hint = curr_shell.size_hint()

        if size_hint == (-1, -1):
            size_hint = curr_shell.min_size()
        
        width_hint, height_hint = size_hint

        # Compute the size of the tab so we can offset the size hint.
        # On Windows, the return value of the height function is a hard
        # coded default of 20. This is close for the standard font but
        # will probably break down with different fonts or icons. I've
        # found no other way to measure the height of the tab bar.
        tab_size = wx.RendererNative.Get().GetHeaderButtonHeight(widget)

        # Offset the size hint by the tab bar size
        style = widget.GetWindowStyle()
        if (style & wx.NB_TOP) or (style & wx.NB_BOTTOM):
            height_hint += tab_size
        else:
            width_hint += tab_size
        
        width_hint = max(width_hint, 200)

        return (width_hint, height_hint)

    #--------------------------------------------------------------------------
    # Event Handlers 
    #--------------------------------------------------------------------------
    def _on_page_changed(self, event):
        """ The event handler for the page change event of the underlying 
        control. Synchronizes the index of the shell object.

        """
        event.Skip()
        # Use event.GetSelection since widget.GetSelection returns the
        # wrong value during this event handler.
        self.shell_obj._selected_index = event.GetSelection()

    #--------------------------------------------------------------------------
    # Widget Update Methods 
    #--------------------------------------------------------------------------
    def set_selected_index(self, index):
        """ Sets the current index of the tab widget. This is overridden
        from the parent class.

        """
        if index != -1:
            self.widget.SetSelection(index)
    
    def update_tabs(self):
        """ Populates the notebook with the child notebook pages. This
        is an overridden parent class method which sets the title of
        of notebook pages properly.

        """
        # FIXME: there should be a more efficient way to do this, but 
        # for now just remove all present widgets and add the current 
        # ones. If we use DeleteAllPages(), then the child widgets would
        # be destroyed, which is not the behavior we want.
        widget = self.widget
        shell = self.shell_obj
        
        while widget.GetPageCount():
            widget.RemovePage(0)
        
        selected = shell.selected_tab
        for idx, tab in enumerate(shell.tabs):
            widget.AddPage(tab.toolkit_widget, tab.title)
            if tab is selected:
                widget.SetSelection(idx)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from traits.api import Bool, Instance

from .constraints_widget import (
    ConstraintsWidget, AbstractTkConstraintsWidget,
)

from ..exceptions import ShellExceptionContext, ShellNotificationContext


class AbstractTkControl(AbstractTkConstraintsWidget):
    """ The abstract toolkit Control interface.

    """
    pass


class Control(ConstraintsWidget):
    """ The base class of all leaf widgets in Enaml.

    """
    #: A boolean which indicates whether an exception was raised through 
    #: user interaction or setting a value trait on the Control.
    error = Bool(False)

    #: The exception object that was raised to create the error state.
    exception = Instance(Exception)

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkControl)

    def capture_exceptions(self):
        """ Return a ShellExceptionContext that will capture error state 
        automatically.
        
        """
        return ShellExceptionContext(self)
    
    def capture_notification_exceptions(
        self, handler=None, reraise_exceptions=True, main=False, locked=False):
        """ Return a ShellNotificationContext that will set error state 
        automatically, including TraitErrors in listeners.
        
        """
        res = ShellNotificationContext(
            self, handler, reraise_exceptions, main, locked,
        )
        return res




# -*- coding: UTF-8 -*-
#------------------------------------------------------------------------------
#  file: fields.py
#  License: LICENSE.TXT
#  Author: Ioannis Tziakos
#
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import collections
import re

from line_functions import add_indent, is_empty, remove_indent, replace_at


class Field(collections.namedtuple('Field', ('name','signature','desc'))):
    """ A docstring field.

    The class is based on the nametuple class and represents the logic
    to check, parse and refactor a docstring field.

    Attributes
    ----------
    name : str
        The name if the field, usssualy the name of a parameter of atrribute.

    signature : str
        The signature of the field. Commonly is the class type of an argument
        or the signature of a function.

    desc : str
        The description of the field. Given the type of the field this is a
        single paragraph or a block of rst source.

    """

    @classmethod
    def is_field(cls, line, indent=''):
        """ Check if the line is a field header.
        """
        regex = indent + r'\*?\*?\w+\s:(\s+|$)'
        match = re.match(regex, line)
        return match

    @classmethod
    def parse(cls, lines):
        """Parse a field definition for a set of lines.

        The field is assumed to be in one of the following formats::

            <name> : <type>
                <description>

        or::

            <name> :
                <description>

        or::

            <name>
                <description>

        Arguments
        ---------
        lines :
            docstring lines of the field without any empty lines before or
            after.

        Returns
        -------
        field :
            Field or subclass of Field

        """
        header = lines[0].strip()
        if ' :' in header:
            arg_name, arg_type = re.split('\s\:\s?', header, maxsplit=1)
        else:
            arg_name, arg_type = header, ''
        if len(lines) > 1:
            lines = [line.rstrip() for line in lines]
            return cls(arg_name.strip(), arg_type.strip(), lines[1:])
        else:
            return cls(arg_name.strip(), arg_type.strip(), [''])


    def to_rst(self, indent=4):
        """ Outputs field in rst as an itme in a definition list.

        Arguments
        ---------
        indent : int
            The indent to use for the decription block.


        Returns
        -------
        lines : list
            A list of string lines of formated rst.

        Example
        -------

        >>> Field('Ioannis', 'Ιωάννης', 'Is the greek guy.')
        >>> print Field.to_rst()
        Ioannis (Ιωάννης)
            Is the greek guy.

        """
        lines = []
        header = '{0} ({1})'.format(self.name, self.signature)
        lines.append(header)
        lines += add_indent(self.desc, indent)
        return lines

class AttributeField(Field):
    """ Field for the argument function docstrings """

    def to_rst(self, indent=4):
        """ Outputs field in rst using the ``:param:`` role.

        Arguments
        ---------
        indent : int
            The indent to use for the decription block.

        Example
        -------

        >>> Field('indent', 'int', 'The indent to use for the decription block.')
        >>> print Field.to_rst()
        :param indent: The indent to use for the description block
        :type indent: int

        """
        lines = []
        _type = self.signature
        annotation = '{0}    :annotation: = {1}'
        type_str = '' if is_empty(_type) else annotation.format(indent * ' ', _type)
        directive = '{0}.. attribute:: {1}'
        lines += [directive.format(indent * ' ', self.name), type_str]
        if type_str != '':
            lines.append('')
        lines += self.desc
        lines.append('')
        return lines


class ArgumentField(Field):
    """ Field for the argument function docstrings """

    def to_rst(self, indent=4):
        """ Outputs field in rst using the ``:param:`` role.

        Arguments
        ---------
        indent : int
            The indent to use for the decription block.

        Example
        -------

        >>> Field('indent', 'int', 'The indent to use for the decription block.')
        >>> print Field.to_rst()
        :param indent: The indent to use for the description block
        :type indent: int

        """
        lines = []
        name = self.name.replace('*','\*')  # Fix cases like *args and **kwargs
        indent_str = ' ' * indent
        param_str = '{0}:param {1}: {2}'.format(indent_str, name, self.desc[0].strip())
        type_str = '{0}:type {1}: {2}'.format(indent_str, name, self.signature)
        lines.append(param_str)
        lines += self.desc[1:]
        if len(self.signature) > 0:
            lines.append(type_str)
        return lines

class ListItemField(Field):
    """ Field that in rst is formated as an item in the list ignoring any
    field.type information.

    """

    def to_rst(self, indent=4, prefix=''):
        """ Outputs field in rst using as items in an list.

        Arguments
        ---------
        indent : int
            The indent to use for the decription block.

        prefix : str
            The prefix to use. For example if the item is part of a numbered
            list then ``prefix='# '``.

        Example
        -------


        Note
        ----
        The field descrption is reformated into a line.

        """
        indent_str = ' ' * indent
        rst_pattern = '{0}{1}**{2}**{3}' if is_empty(self.desc[0]) else \
                       '{0}{1}**{2}** -- {3}'
        description = '' if is_empty(self.desc[0]) else \
                      ' '.join(remove_indent(self.desc))
        return [rst_pattern.format(indent_str, prefix, self.name, description)]


class ListItemWithTypeField(Field):
    """ Field for the return section of the function docstrings """
    def to_rst(self, indent=4, prefix=''):
        indent_str = ' ' * indent
        _type = '' if self.signature == '' else '({0})'.format(self.signature)
        rst_pattern = '{0}{1}**{2}** {3}{4}' if is_empty(self.desc[0]) else \
                       '{0}{1}**{2}** {3} -- {4}'
        description = '' if is_empty(self.desc[0]) else \
                    ' '.join(remove_indent(self.desc))
        return [rst_pattern.format(indent_str, prefix, self.name, _type, description)]


class FunctionField(Field):
    """ A field that represents a function """

    @classmethod
    def is_field(cls, line, indent=''):
        regex = indent + r'\w+\(.*\)\s*'
        match = re.match(regex, line)
        return match

    def to_rst(self, length, first_column, second_column):
                split_result = re.split('\((.*)\)', self.name)
                method_name = split_result[0]
                method_text = ':meth:`{0} <{1}>`'.format(self.name, method_name)
                summary = ' '.join([line.strip() for line in self.desc])
                line = ' ' * length
                line = replace_at(method_text, line, first_column)
                line = replace_at(summary, line, second_column)
                return [line]

MethodField = FunctionField

#------------------------------------------------------------------------------
#  Functions to work with fields
#------------------------------------------------------------------------------

def max_name_length(method_fields):
    """ Find the max length of the function name in a list of method fields.

    Arguments
    ---------
    fields : list
        The list of the parsed fields.

    """
    return max([field[0].find('(') for field in method_fields])

def max_header_length(fields):
    """ Find the max length of the header in a list of fields.

    Arguments
    ---------
    fields : list
        The list of the parsed fields.

    """
    return max([len(field[0]) for field in fields])

def max_desc_length(fields):
    """ Find the max length of the description in a list of fields.

    Arguments
    ---------
    fields : list
        The list of the parsed fields.

    """
    return max([len(' '.join([line.strip() for line in field[2]]))
                for field in fields])



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .enaml_test_case import EnamlTestCase, required_method


class TestSpinBox(EnamlTestCase):
    """ Logic for testing spin boxes.

    Tooklit testcases need to provide the following methods

    Abstract Methods
    ----------------
    def get_value(self, widget)
        Get a spin box's value.

    def get_low(self, widget)
        Get a spin box's minimum value.

    get_high(self, widget)
        Get a spin box's maximum value.

    get_step(self, widget)
        Get a spin box's step size.

    get_wrap(self, widget)
        Check if a spin box wraps around at the edge values.

    get_text(self, widget)
        Get the text displayed in a spin box.

    spin_up_event(self, widget)
        Simulate a click on the 'up' spin button.

    spin_down_event(self, widget)
        Simulate a click on the 'down' spin button.

    """
    def setUp(self):
        """ Set up before the spin box tests.

        """

        enaml_source = """
enamldef MainView(MainWindow):
    attr events
    SpinBox:
        name = 'spinbox'
        low = -10
        high = 21
        step = 2
        value = -4
        wrap = True
"""

        self.events = []
        self.view = self.parse_and_create(enaml_source, events=self.events)
        self.component = self.component_by_name(self.view, 'spinbox')
        self.widget = self.component.toolkit_widget

    def test_initial_attributes(self):
        """ Compare the Enaml SpinBox's attributes with its toolkit widget.

        """
        component = self.component
        widget = self.widget
        self.assertEqual(self.get_value(widget), component.value)
        self.assertEqual(self.get_low(widget), component.low)
        self.assertEqual(self.get_high(widget), component.high)
        self.assertEqual(self.get_step(widget), component.step)
        self.assertEqual(self.get_wrap(widget), component.wrap)

    def test_change_low(self):
        """ Update the spin box's minimum value.

        """
        component = self.component
        new_low = component.low - 1
        component.low = new_low
        component_low = component.low
        widget_low = self.get_low(self.widget)
        self.assertEqual(component_low, new_low)
        self.assertEqual(widget_low, component_low)

    def test_change_high(self):
        """ Update the spin box's maximum value.

        """
        component = self.component
        new_high = component.high + 1
        component.high = new_high
        component_high = component.high
        widget_high = self.get_high(self.widget)
        self.assertEqual(component_high, new_high)
        self.assertEqual(widget_high, component_high)

    def test_step_up(self):
        """ Simulate a press of the spin box's 'up' button.

        """
        component = self.component
        widget = self.widget
        old_widget_value = self.get_value(widget)
        widget_step = self.get_step(widget)
        self.spin_up_event(widget)
        new_widget_value = self.get_value(widget)
        self.assertEqual(new_widget_value, old_widget_value + widget_step)
        self.assertEqual(component.value, new_widget_value)

    def test_step_down(self):
        """ Simulate a press of the spin box's 'down' button.

        """
        component = self.component
        widget = self.widget
        old_widget_value = self.get_value(widget)
        widget_step = self.get_step(widget)
        self.spin_down_event(widget)
        new_widget_value = self.get_value(widget)
        self.assertEqual(new_widget_value, old_widget_value - widget_step)
        self.assertEqual(component.value, new_widget_value)

    def test_wrap_top(self):
        """ Check that a spin box wraps appropriately, from top to bottom.

        """
        widget = self.widget
        component = self.component
        component.value = component.high
        self.spin_up_event(self.widget)
        self.assertEqual(component.value, component.low)
        self.assertEqual(component.value, self.get_value(widget))

    def test_wrap_bottom(self):
        """ Check that a spin box wraps appropriately, from bottom to top.

        """
        widget = self.widget
        component = self.component
        component.value = component.low
        self.spin_down_event(widget)
        self.assertEqual(component.value, component.high)
        self.assertEqual(component.value, self.get_value(widget))

    def test_no_wrap_top(self):
        """ Check that a spin box doesn't wrap from top to bottom.

        """
        widget = self.widget
        component = self.component
        high = component.high
        component.wrap = False
        component.value = high
        self.spin_up_event(widget)
        self.assertEqual(component.value, high)
        self.assertEqual(component.value, self.get_value(widget))

    def test_no_wrap_bottom(self):
        """ Check that a spin box doesn't wrap from bottom to top.

        """
        widget = self.widget
        component = self.component
        low = component.low
        component.wrap = False
        component.value = low
        self.spin_down_event(widget)
        self.assertEqual(component.value, low)
        self.assertEqual(component.value, self.get_value(widget))

    def test_text_display(self):
        """ Check the displayed text.

        """
        self.assertEqual(self.get_text(self.widget), '-4')

    #--------------------------------------------------------------------------
    # Abstract methods
    #--------------------------------------------------------------------------
    @required_method
    def get_value(self, widget):
        """ Get a spin box's value.

        """
        pass

    @required_method
    def get_low(self, widget):
        """ Get a spin box's minimum value.

        """
        pass

    @required_method
    def get_high(self, widget):
        """ Get a spin box's maximum value.

        """
        pass

    @required_method
    def get_step(self, widget):
        """ Get a spin box's step size.

        """
        pass

    @required_method
    def get_wrap(self, widget):
        """ Check if a spin box wraps around at the edge values.

        """
        pass

    @required_method
    def get_text(self, widget):
        """ Get the text displayed in a spin box.

        """
        pass

    @required_method
    def spin_up_event(self, widget):
        """ Simulate a click on the 'up' spin button.

        """
        pass

    @required_method
    def spin_down_event(self, widget):
        """ Simulate a click on the 'down' spin button.

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Int, Instance, Property, TraitError, on_trait_change

from .control import Control, AbstractTkControl

from ..core.trait_types import Bounded


class AbstractTkProgressBar(AbstractTkControl):

    @abstractmethod
    def shell_value_changed(self, value):
        raise NotImplementedError

    @abstractmethod
    def shell_minimum_changed(self, minimum):
        raise NotImplementedError

    @abstractmethod
    def shell_maximum_changed(self, maximum):
        raise NotImplementedError


class ProgressBar(Control):
    """ A progress bar.

    """
    #: The minimum value that the progress bar can take. Extra checks
    #: take place to make sure that the user does not programmatically 
    #: set :attr:`minimum` > :attr:`maximum`.
    minimum = Property(Int, depends_on='_minimum')

    #: The internal min value storage
    _minimum = Int(0)

    #: The maximum value that the progress bar can take. Extra checks 
    #: take place to make sure that the user does not programmatically 
    #: set :attr:`minimum` > :attr:`maximum`.
    maximum = Property(Int, depends_on='_maximum')

    #: The internal max value storage
    _maximum = Int(100)

    #: The current value. Default is the minimum value. The value is 
    #: bounded between :attr:`minimum` and :attr:`maximum`. Changing 
    #: the boundary attributes might result in an update of :attr:`value` 
    #: to fit in the new range. Attempts to assign a value outside of 
    #: these bounds will result in a TraitError.
    value = Bounded(0, low='minimum', high='maximum')

    #: The percentage completed, rounded to an integer. This is a 
    #: readonly property for convenient use by other Components.
    percentage = Property(Int, depends_on=['_minimum', '_maximum', 'value'])

    #: How strongly a component hugs it's content. ProgressBars expand 
    #: to fill the available horizontal space by default.
    hug_width = 'ignore'

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkProgressBar)

    #--------------------------------------------------------------------------
    # Property methods
    #--------------------------------------------------------------------------
    def _set_minimum(self, value):
        """ The property setter for :attr:`minimum`. Addtional checks are 
        applied to make sure that :attr:`minimum` < :attr:`maximum`

        """
        if value > self.maximum:
            msg = ('The minimum value of ProgressBar should be smaller than '
                   'the current maximum value({0}), but a value of {1} was '
                   'given')
            msg = msg.format(self.maximum, value)
            raise TraitError(msg)
        self._minimum = value

    def _set_maximum(self, value):
        """ The property setter for :attr:`maximum`. Addtional checks are 
        applied to make sure that :attr:`minimum` < :attr:`maximum`

        """
        if value < self.minimum:
            msg = ('The maximum value of ProgressBar should be larger than '
                   'the current minimum value({0}), but a value of {1} was '
                   'given')
            msg = msg.format(self.minimum, value)
            raise TraitError(msg)
        self._maximum = value

    def _get_maximum(self):
        """ The property getter for the ProgressBar maximum.

        """
        return self._maximum

    def _get_minimum(self):
        """ The property getter for the ProgressBar minimum.

        """
        return self._minimum

    def _get_percentage(self):
        """ The property getter for the ProgressBar percentage.

        """
        minimum = self.minimum
        maximum = self.maximum
        value = self.value
        dy = maximum - minimum
        if dy == 0:
            res = 0
        elif value == maximum:
            res = 100
        else:
            dx = float(value - minimum)
            res = int(round(100.0 * dx / dy))
            # We already excluded the case where the value was exactly the 
            # maximum, so we can't really be at 100%, so round this down to 
            # 99% if necessary.
            res = min(res, 99)
        return res

    @on_trait_change('minimum, maximum')
    def _adapt_value(self):
        """ Adapt the value to the boundaries

        """
        if self.initialized:
            self.value = min(max(self.value, self.minimum), self.maximum)




# -*- coding: UTF-8 -*-
#------------------------------------------------------------------------------
#  file: base_doc.py
#  License: LICENSE.TXT
#  Author: Ioannis Tziakos
#
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import re

from fields import Field
from line_functions import is_empty, get_indent, fix_backspace

#------------------------------------------------------------------------------
#  Classes
#------------------------------------------------------------------------------

class BaseDoc(object):
    """Base abstract docstring refactoring class.

    The class' main purpose is to parse the dosctring and find the
    sections that need to be refactored. It also provides a number of
    methods to help with the refactoring. Subclasses should provide
    the methods responsible for refactoring the sections.

    Attributes
    ----------
    docstring : list
        A list of strings (lines) that holds docstrings

    index : int
        The current zero-based line number of the docstring that is
        proccessed.

    verbose : bool
        When set the class prints a lot of info about the proccess
        during runtime.

    headers : dict
        The sections that the class refactors. Each entry in the
        dictionary should have as key the name of the section in the
        form that it appears in the docstrings. The value should be
        the postfix of the method, in the subclasses, that is
        responsible for refactoring (e.g. {'Methods': 'method'}).

    """

    def __init__(self, lines, headers = None, verbose=False):
        """ Initialize the class

        The method setups the class attributes and starts parsing the
        docstring to find and refactor the sections.

        Arguments
        ---------
        lines : list of strings
            The docstring to refactor

        headers : dict
            The sections for which the class has custom refactor methods.
            Each entry in the dictionary should have as key the name of
            the section in the form that it appears in the docstrings.
            The value should be the postfix of the method, in the
            subclasses, that is responsible for refactoring (e.g.
            {'Methods': 'method'}).

        verbose : bool
            When set the class logs info about the proccess
            during runtime.

        """
        try:
            self._docstring = lines.splitlines()
        except AttributeError:
            self._docstring = lines
        self.verbose = verbose
        self.headers = {} if headers is None else headers
        self.index = 0
        self.parse()

    def parse(self):
        """ Parse the docstring.

        The docstring is parsed for sections. If a section is found then
        the corresponding refactoring method is called.

        """
        self.index = 0
        self.seek_to_next_non_empty_line()
        while not self.eol:
            header = self.is_section()
            if header:
                self.remove_if_empty(self.index + 2)  # Remove space after header
                self._refactor(header)
            else:
                self.index += 1
                self.seek_to_next_non_empty_line()

    def _refactor(self, header):
        """Call the heading refactor method.

        The name of the refctoring method is constructed using the form
        _refactor_<header>. Where <header> is the value corresponding to
        ``self.headers[header]``. If there is no custom method for the
        section then the self._refactor_header() is called with the
        found header name as input.

        """
        refactor_postfix = self.headers.get(header, 'header')
        method_name = ''.join(('_refactor_', refactor_postfix))
        method = getattr(self, method_name)
        method(header)

    def _refactor_header(self, header):
        """ Refactor the header section using the rubric directive.

        The method has been tested and supports refactoring single word
        headers, two word headers and headers that include a backslash
        ''\''.

        Arguments
        ---------
        header : string
            The header string to use with the rubric directive.

        """
        index = self.index
        indent = get_indent(self.peek())
        self.remove_lines(index, 2)
        descriptions = []
        header = fix_backspace(header)
        descriptions += [indent + '.. rubric:: {0}'.format(header), '']
        self.insert_lines(descriptions, index)
        self.index += len(descriptions)
        return descriptions


    def extract_fields(self, indent='', field_type=None):
        """Extract the fields from the docstring

        Parse the fields in the description of a section into tuples of
        name, type and description in a list of strings. The parsed lines
        are also removed from original list.

        Arguments
        ---------
        indent : str, optional
            the indent argument is used to make sure that only the lines
            with the same indent are considered when checking for a
            field header line. The value is used to define the field
            checking function.

        field_check : function
            Optional function to use for checking if the next line is a
            field. The signature of the function is ``foo(line)`` and it
            should return ``True`` if the line contains a valid field
            The default function is checking for fields of the following
            formats::

                <name> : <type>

            or::

                <name> :

            Where the name has to be one word.

        Returns
        -------
        parameters : list of tuples
            list of parsed parameter tuples as returned from the
            :meth:`~BaseDocstring.parse_field` method.

        """
        field_type = Field if (field_type is None) else field_type
        is_field = field_type.is_field
        fields = []
        while (not self.eol) and (is_field(self.peek(), indent) or
                                  is_field(self.peek(1), indent)):
            self.remove_if_empty(self.index)
            field_block = self.get_next_block()
            field = field_type.parse(field_block)
            fields.append(field)
        return fields

    def get_next_block(self):
        """ Get the next field block from the docstring.

        The method reads the next block in the docstring. The first line
        assumed to be the field header and the following lines to belong to
        the description::

            <header line>
                <descrition>

        The end of the field is designated by a line with the same indent
        as the field header or two empty lines are found in sequence. Thus,
        there are two valid field layouts:

        1. No lines between fields::

            <field1>
                <description1>
            <fieldd2>
                <description2>

        2. One line between fields::

            <field1>
                <description1>

            <field2>
                <description2>

        """
        start = self.index
        field_header = self.read()
        indent = get_indent(field_header) + ' '
        field = [field_header]
        while (not self.eol):
            peek_0 = self.peek()
            peek_1 = self.peek(1)
            if (is_empty(peek_0) and (not peek_1.startswith(indent))) \
                or \
                ((not is_empty(peek_0)) and (not peek_0.startswith(indent))):
                break
            else:
                line = self.read()
                field.append(line.rstrip())

        self.remove_lines(start, len(field))
        self.index = start
        return field

    def is_section(self):
        """Check if the line defines a section.

        """
        if self.eol:
            return False

        header = self.peek()
        line2 = self.peek(1)
        if self.verbose:
            print 'current line is: {0} at index {1}'.format(header, self.index)

        # check for underline type format
        underline = re.match(r'\s*\S+\s*\Z', line2)
        if underline is None:
            return False
        # is the nextline an rst underline?
        striped_header = header.rstrip()
        expected_underline1 = re.sub(r'[A-Za-z\\]|\b\s', '-', striped_header)
        expected_underline2 = re.sub(r'[A-Za-z\\]|\b\s', '=', striped_header)
        if ((underline.group().rstrip() == expected_underline1) or
            (underline.group().rstrip() == expected_underline2)):
            return header.strip()
        else:
            return False

    def insert_lines(self, lines, index):
        """ Insert refactored lines

        Arguments
        ---------
        new_lines : list
            The list of lines to insert

        index : int
            Index to start the insertion
        """
        docstring = self.docstring
        for line in reversed(lines):
            docstring.insert(index, line)

    def seek_to_next_non_empty_line(self):
        """ Goto the next non_empty line

        """
        docstring = self.docstring
        for line in docstring[self.index:]:
            if not is_empty(line):
                break
            self.index += 1


    def get_next_paragraph(self):
        """ Get the next paragraph designated by an empty line.

        """
        docstring = self.docstring
        lines = []
        start = self.index
        while (not self.eol) and (not is_empty(self.peek())):
            line = self.read()
            lines.append(line)
        del docstring[start:self.index]
        return lines

    def read(self):
        """ Return the next line and advance the index.

        """
        index = self.index
        line = self._docstring[index]
        self.index += 1
        return line

    def remove_lines(self, index, count=1):
        """ Removes the lines for the docstring

        """
        docstring = self.docstring
        del docstring[index:(index + count)]

    def remove_if_empty(self, index=None):
        """ Remove the line from the docstring if it is empty.

        """
        if is_empty(self.docstring[index]):
            self.remove_lines(index)

    def peek(self, ahead=0):
        """ Peek ahead a number of lines

        The function retrieves the line that is ahead of the current
        index. If the index is at the end of the list then it returns an
        empty string.

        Arguments
        ---------
        ahead : int
            The number of lines to look ahead.


        """
        position = self.index + ahead
        try:
            line = self.docstring[position]
        except IndexError:
            line = ''
        return line

    @property
    def eol(self):
        return self.index >= len(self.docstring)

    @property
    def docstring(self):
        """ Get the docstring lines.

        """
        return self._docstring

